library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

--  Uncomment the following lines to use the declarations that are
--  provided for instantiating Xilinx primitive components.
--library UNISIM;
--use UNISIM.VComponents.all;

entity clock is		   
    Port ( RESET : in std_logic;
           MCKI : in std_logic;
           MCKO,SCKO : out std_logic;
		 BCKI,LRCKI,DATAI : in std_logic;
           BCKO,LRCKO,DATAO : out std_logic;
		 SPOE : in std_logic;  
		 AVR_LRCKI,AVR_LRCKOR,AVR_LRCKOL : out std_logic;
		 AVR_DATA : inout std_logic_vector(15 downto 0));
end clock;


architecture Behavioral of clock is
	signal	c :  integer range 0 to 3;
	signal	c1 :  std_logic_vector(7 downto 0);
	signal	c2 :  std_logic;
	signal	c32fs : std_logic;
	signal	c44k : std_logic;

	signal	lrcko_1 : std_logic;
	signal	lrcki_1 : std_logic;
	signal	ps_data : std_logic_vector(15 downto 0);

	signal	latch_data_in,sp_data : std_logic_vector(15 downto 0);
begin
	MCKO <= MCKI;	
	SCKO <= c2;		
	BCKO <= not c32fs ;
	LRCKO <= lrcko_1 ;
	DATAO <= ps_data(15);
	AVR_LRCKI <= LRCKI;

	c32fs<=c1(2);
	c44k<=c1(7);

	process (c1(7),c1(6),c1(5)) begin
		if ((c1(7)='0') and (c1(6)='1')) then
			AVR_LRCKOL <= '1';
		else
			AVR_LRCKOL <= '0';
		end if;
		if ((c1(7)='1') and (c1(6)='1')) then
			AVR_LRCKOR <= '1';
		else
			AVR_LRCKOR <= '0';
		end if;
	end process;

	process (MCKI,RESET) begin
		if (RESET='0') then
			c<=0;
		   	c1<="00000000";
			c2<='0';
		elsif (MCKI'event and MCKI='1') then
			c <= c+1;
			c2<=not c2;
			if (c = 2) then  
				c1<=c1+1;
				c<=0;
			end if;
		end if; 
	end process;

	process (c32fs,RESET) begin
		if (RESET='0') then
			ps_data<="0000000000000000";
			AVR_DATA<="ZZZZZZZZZZZZZZZZ";
			lrcko_1<='0';
		elsif (c32fs'event and c32fs='1') then
			lrcko_1 <= c44k;
			if ((c44k xor (not lrcko_1))='0') then
				ps_data <= AVR_DATA;
			else
				ps_data(15 downto 1) <= ps_data(14 downto 0);
			end if;
		end if;
	end process;

	process (BCKI,RESET) begin
		if (RESET='0') then
			sp_data<="0000000000000000";
			lrcki_1<='0';
		elsif (BCKI'event and BCKI='1') then
			sp_data(15 downto 0) <= sp_data(14 downto 0) & DATAI;
			lrcki_1 <= LRCKI;
			if ((LRCKI xor (not lrcki_1))='0') then
				latch_data_in <= sp_data;
			end if;
		end if;
	end process;

	process (SPOE) begin
		if (SPOE='0') then 
			AVR_DATA<="ZZZZZZZZZZZZZZZZ";
		else
			AVR_DATA<=latch_data_in;
		end if;
	end process;

end Behavioral;
